@@ -19,18 +19,27 @@
 #include <iostream>
 #include <QRegExp>
 #include <QMessageService>
-#include <QSqlQuery>
+#include <QDateTime>
+#include <QUuid>
+#include <time.h>
 
 using namespace QtMobility;
 
 SmsSender::SmsSender(int argc, char *argv[]) :
         QCoreApplication(argc,argv)
 {
-    m_Db = NULL;
     m_LogFile = NULL;
     m_Save = true;
+    m_El = NULL;
     m_Send = false;
     m_Flash = false;
+    //For delivery report to work the program should be listening until the
+    //message is delivered:
+    //+CDS: 6,<the SMS ref num above>,,,<date & time>,<date & time>,0 or 48
+    // last character - 0 being successful, 48 delivery failed
+    //This is not possible with a CLI program (should exit as the sms is sent).
+    //m_DeliveryReport = false;
+    m_Validity = 167;
     m_Argc = argc - 1;
     for (int i=1; i<argc; i++)
         m_Argv.append(argv[i]);
@@ -43,8 +52,8 @@ SmsSender::~SmsSender()
         delete m_LogFile;
     }
 
-    if (m_Db != NULL)
-        m_Db->close();
+    if (m_El != NULL)
+        g_object_unref(m_El);
 }
 
 void SmsSender::consoleOutput(QString output, bool newline)
@@ -68,12 +77,18 @@ void SmsSender::printUsage()
     consoleOutput("Options:");
     consoleOutput("  -c         : compose the message with conversation");
     consoleOutput("  -C         : send the message with conversation");
+    consoleOutput("");
+    consoleOutput("Following options are valid only if used without -c or -C");
     consoleOutput("  -l logfile : log pnatd messages to logfile");
-    consoleOutput("               This option is useless if used with -c or -C");
     consoleOutput("  -s         : don't save message in the sent folder");
-    consoleOutput("               This option is useless if used with -c or -C");
     consoleOutput("  -f         : send a flash sms");
-    consoleOutput("               This option is useless if used with -c or -C");
+    //consoleOutput("  -r         : request delivery report");
+    consoleOutput("  -v VP      : set the sms validity period (0-255)");
+    consoleOutput("               Default value is 167 (1 day)");
+    consoleOutput("                 0 - 143 (VP + 1) x 5 mins");
+    consoleOutput("               144 - 167 12 Hours + ((VP-143) x 30 mins)");
+    consoleOutput("               168 - 196 (VP-166) x 1 day");
+    consoleOutput("               197 - 255 (VP-192) x 1 week");
 }
 
 int SmsSender::exec()
@@ -109,6 +124,10 @@ int SmsSender::exec()
                 m_Save = false;
             else if (param == "-f")
                 m_Flash = true;
+            /*else if (param == "-r")
+                m_DeliveryReport = true;*/
+            else if (param == "-v")
+                currArg = SmsSender::argValidity;
             else{
                 consoleOutput(QString("ERR: unrecognized argument: %1").arg(param));
                 return errArguments;
@@ -139,6 +158,17 @@ int SmsSender::exec()
                             return errLogfileOpenFailed;
                         }
                     }
+                    break;
+                case SmsSender::argValidity:
+                    {
+                        bool ok = false;
+                        m_Validity = param.toInt(&ok);
+                        if (!ok || m_Validity < 0 || m_Validity > 255){
+                            consoleOutput(QString("WARNING: Invalid validity period %1").arg(m_Validity));
+                            m_Validity = 167;
+                        }
+                    }
+                    break;
                 default:
                     break;
             }
@@ -161,18 +191,8 @@ int SmsSender::exec()
     }
 
     if (ok){
-        m_Db = new QSqlDatabase();
-        *m_Db = QSqlDatabase::addDatabase( "QSQLITE" );
-        m_Db->setDatabaseName("/home/user/.rtcom-eventlogger/el-v1.db");
-        if (!m_Db->open()){
-            consoleOutput("WARNING: error opening events database");
-            delete m_Db;
-            m_Db = NULL;
-        }else{
-            QSqlQuery qry;
-            if (qry.prepare("PRAGMA synchronous=OFF;"))
-                qry.exec();
-        }
+        if (m_Save)
+            m_El = rtcom_el_new();
 
         QStringList messageBlocks;
         if (m_Compose || message.length() <= 160)
@@ -213,7 +233,6 @@ int SmsSender::exec()
     return res;
 }
 
-
 bool SmsSender::compose(QString number, QString messageText)
 {
     //Implementation using QtMobility (opens conversation after the message is moved to the sent folder):
@@ -267,8 +286,16 @@ bool SmsSender::send(QString number, QString messageText)
             consoleOutput(".", false);
     }
 
-    if (res && m_Flash){
-        if (!writeCmd(pty, QString("AT+CSMP=17,167,0,16\r"), "OK")){
+    if (res){
+        int opt = 17;
+        /*if (m_DeliveryReport)
+            opt += 32;*/
+
+        int dcs = 0;
+        if (m_Flash)
+            dcs = 16;
+
+        if (!writeCmd(pty, QString("AT+CSMP=%1,%2,0,%3\r").arg(opt).arg(m_Validity).arg(dcs), "OK")){
             consoleOutput(QString("\nERR: Failed to send at+csmp command"));
             res = false;
         }else
@@ -293,12 +320,12 @@ bool SmsSender::send(QString number, QString messageText)
 
     if (res){
         res = writeCmd(pty, QString("%1").arg(QChar(26)).toLatin1(), "OK", -1);
-        if (!res)
+        if (!res){
             consoleOutput("FAILED");
-        else{
+        }else{
             consoleOutput("OK");
             if (m_Save){
-                if (m_Db != NULL && !saveMsgToSentFolder(number, messageText))
+                if (!saveMsgToSentFolder(number, messageText))
                     consoleOutput("WARNING: Failed to save sent sms");
             }
         }
@@ -333,39 +360,33 @@ bool SmsSender::writeCmd(Pty* p, QString cmd, QString okReply, int msectimeout)
 }
 
 bool SmsSender::saveMsgToSentFolder(QString number, QString message)
-{    
-    //This isn't the best way, it should really be replaced with some API.
-    //ATM QtMobility doesn't have a method to save a message in a folder.
-    message.replace("'", "''");
+{
+    bool result = false;
     QString partNumber = number.right(7);
 
-    QSqlQuery qry;
-    if (qry.prepare("BEGIN IMMEDIATE TRANSACTION;"))
-        qry.exec();
+    RTComElEvent* ev = rtcom_el_event_new();
 
-    QString sql = QString("insert into Events (service_id, event_type_id, storage_time, start_time, end_time, is_read, flags, \
-                                               bytes_sent, bytes_received, local_uid, local_name, remote_uid, free_text, group_uid, outgoing) \
-                           values (3, 7, strftime('%s', 'now'), strftime('%s', 'now'), 0, 1, 0, 0, 0, 'ring/tel/ring', '<SelfHandle>', '%1', '%2', '%3', 1)")
-                           .arg(number)
-                           .arg(message)
-                           .arg(partNumber);
-    if (!qry.prepare(sql)){
-        if (qry.prepare("ROLLBACK;"))
-            qry.exec();
-        return false;
+    RTCOM_EL_EVENT_SET_FIELD(ev, service, g_strdup(QString("RTCOM_EL_SERVICE_SMS").toLatin1().data()));
+    RTCOM_EL_EVENT_SET_FIELD(ev, event_type, g_strdup(QString("RTCOM_EL_EVENTTYPE_SMS_OUTBOUND").toLatin1().data()));
+    RTCOM_EL_EVENT_SET_FIELD(ev, start_time, time(NULL));
+    RTCOM_EL_EVENT_SET_FIELD(ev, end_time, time(NULL));
+    RTCOM_EL_EVENT_SET_FIELD(ev, is_read, TRUE);
+    RTCOM_EL_EVENT_SET_FIELD(ev, remote_ebook_uid, 0);
+    RTCOM_EL_EVENT_SET_FIELD(ev, local_uid, g_strdup(QString("ring/tel/ring").toLatin1().data()));
+    RTCOM_EL_EVENT_SET_FIELD(ev, local_name, g_strdup(QString("<SelfHandle>").toLatin1().data()));
+    RTCOM_EL_EVENT_SET_FIELD(ev, remote_uid, number.toLatin1().data());
+    RTCOM_EL_EVENT_SET_FIELD(ev, remote_name, g_strdup(QString("").toLatin1().data()));
+    RTCOM_EL_EVENT_SET_FIELD(ev, group_uid, g_strdup(partNumber.toLatin1().data()));
+    RTCOM_EL_EVENT_SET_FIELD(ev, free_text, g_strdup(message.toUtf8().data()));
+
+    int id = rtcom_el_add_event(m_El, ev, NULL);
+    if (id >= 0){
+        QUuid uid = QUuid::createUuid();
+        rtcom_el_add_header(m_El, id, QString("message-token").toLatin1().data(), uid.toString().toLatin1().data(), NULL);
+        result = true;
     }
 
-    if (!qry.exec()){
-        if (qry.prepare("ROLLBACK;"))
-            qry.exec();
-        return false;
-    }
+    rtcom_el_event_free(ev);
 
-    if (qry.prepare("COMMIT;"))
-        qry.exec();
-
-    //QMessageService service;
-    //service.exportUpdates(QMessageAccount::defaultAccount(QMessage::Sms));
-
-    return true;
+    return result;
 }
